--[[
Abilities.lua
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2009 by Horacio Hoyos

This file is part of DHUD3.

    DHUD3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD3.  If not, see <http://www.gnu.org/licenses/>.
]]
local DHUD3 = LibStub("AceAddon-3.0"):GetAddon("DHUD3");
local L = LibStub("AceLocale-3.0"):GetLocale("DHUD3");

local MODNAME = "Abilities";
local Abilities = DHUD3:NewModule(MODNAME, "AceEvent-3.0");

local string_sub = string.sub;
local defaultfont = "Fonts/FRIZQT__.TTF";
local side;
local abilityUpdateInterval = 1;
local abiliPosition = "i";
local abiliOrigin;
local abiliSideBlocked = false;

local RUNETYPE_BLOOD = 1;
local RUNETYPE_UNHOLY = 2;
local RUNETYPE_FROST = 3;
local RUNETYPE_DEATH = 4;

local runeTextures = {
	[RUNETYPE_BLOOD] = "Interface\\AddOns\\DHUD3\\textures\\Blood",
	[RUNETYPE_UNHOLY] = "Interface\\AddOns\\DHUD3\\textures\\Unholy",
	[RUNETYPE_FROST] = "Interface\\AddOns\\DHUD3\\textures\\Frost",
	[RUNETYPE_DEATH] = "Interface\\AddOns\\DHUD3\\textures\\Death",
}

local runeColors = {
	[RUNETYPE_BLOOD] = {1, 0, 0},
	[RUNETYPE_UNHOLY] = {0, 0.5, 0},
	[RUNETYPE_FROST] = {0, 1, 1},
	[RUNETYPE_DEATH] = {0.8, 0.1, 1},
}
local runeMapping = {
	[1] = "BLOOD",
	[2] = "UNHOLY",
	[3] = "FROST",
	[4] = "DEATH",
}

--Local Defaults
local db
local defaults = {
	profile = {
        side = "r";
		scale = 1;
		fontSize = 10;
		tips = true;

		druidCombos = true;
		druidLacerates = true;
		druidLaceratesTimer  = true;
		druidLifebloom  = true;
		druidLifebloomTimer  = true;
		rogueCombos = true;
		warSunders = true;
		warSundersTimer = true;
        colors = {
            full = { 1, 1, 1 },
            med = { 255/255, 125/255, 125/255 },
            low = { 255/255, 65/255, 0 }
        },
		dkColors = {
            full = { 1, 0, 0 },
            med = { 255/255, 228/255, 0 },
            low = { 0, 1, 0 }
        },
    }
}

local optGetter, optSetter, colorGetter, colorSetter
do
	local mod = Abilities
	function optGetter(info)
		local key = info[#info];
		return db[key];
	end

	function optSetter(info, value)
		local key = info[#info];
		db[key] = value;
		mod:Refresh();
	end

    function ColorGetter (info)
        local key1, key2 = info[#info-1], info[#info];
		return db[key1][key2][1], db[key1][key2][2], db[key1][key2][3];
	end
    
    function ColorSetter (info, r, g, b) 
        local key1, key2, key3 = info[#info-2], info[#info-1], info[#info];
        db[key1][key2][1] = r;
        db[key1][key2][2] = g;
        db[key1][key2][3] = b;
        mod:Refresh();
    end

	function selfDisabled()
		return not DHUD3:GetModuleEnabled(MODNAME);
	end
end

--Local Options
local options
local function getOptions()
	if not options then
        --Options table
    	options = {
    		type = "group",
			name = L["Abilities"],
			arg = MODNAME,
			get = optGetter,
			set = optSetter,
    		args = {
                enabled = {
                    type = "toggle",
                    order = 1,
                    name = L["Enable Abilities"],
                    get = function() return DHUD3:GetModuleEnabled(MODNAME) end,
                    set = function(info, value) DHUD3:SetModuleEnabled(MODNAME, value) end,
                },
				general = {
					type = 'group',
					name = L["General"],
					order = 2,
					inline = true,
					disabled  = selfDisabled,
					args = {
						desc = {
							order = 0,
							type = "description",
							name = L["Configure general functionalily"],
						},
						side = {
							type = 'select',
							order = 1,
							name = L["Abilities Side"],
							desc = L["This setting will only be available if the Auras Module is disabled"],
							values = {["r"] = L["Right"], ["l"] = L["Left"]},
							disabled = function() return DHUD3:GetModuleEnabled("Auras"); end,
						},
						tips = {
							type = 'toggle',
							order = 2,
							name = L["Show Abilities tips"],
						},
						fontSize = {
							type = 'range',
							order = 3,
							name = L["Font Size"],
							min = 6,
							max = 25,
							step = 1,
						},
						colors = {
							type = 'group',
							name = L["Border Color"],
							order = 7,
							guiInline  = true,
							get = ColorGetter,
		                    set = ColorSetter,
							args = {
								full = {
									type = 'color',
									name = L["Full"],
									order = 1,
								},
								med = {
									type = 'color',
									name = L["Med"],
									order = 2,
								},
								low = {
									type = 'color',
									name = L["Low"],
									order = 3,
								},
							},
						},
						scale = {
							type = "range",
							order = 6,
							name = L["Set Abilities's scale"],
							min = 0.5,
							max = 1,
							step = 0.05,
						},
					},
				},
				druid = {
					type = 'group',
					name = L["Druid"],
					inline = true,
					disabled = selfDisabled,
					args = {
						druidCombos = {
							type = 'toggle',
							order = 1,
							name = L["Combos"],
							desc = L["Track druid combo points"],
						},
						druidLacerates = {
							type = 'toggle',
							order = 2,
							name = L["Lacerates"],
							desc = L["Track druid lacerates"],
						},
						druidLaceratesTimer = {
							type = 'toggle',
							order = 3,
							name = L["Lacerates Timer"],
							desc = L["Track Druid lacerates expiration"],
							disabled = function() return not db.druidLacerates; end,
						},
						druidLifebloom = {
							type = 'toggle',
							order = 2,
							name = L["Lifebloom"],
							desc = L["Track druid lifebloom"],
						},
						druidLifebloomTimer = {
							type = 'toggle',
							order = 3,
							name = L["Lifebloom Timer"],
							desc = L["Track Druid lifebloom expiration"],
							disabled = function() return not db.druidLifebloom; end,
						},
					},
				},
				rogue = {
					type = 'group',
					name = L["Rogue"],
					inline = true,
					disabled = selfDisabled,
					args = {
						rogueCombos = {
							type = 'toggle',
							order = 1,
							name = L["Combos"],
							desc = L["Track rogue combo points"],
						},
						
					},
				},
				shaman = {
					type = 'group',
					name = L["Shaman"],
					inline = true,
					disabled = selfDisabled,
					args = {
					},
				},
				death = {
					type = 'group',
					name = L["Death Knight"],
					inline = true,
					disabled = selfDisabled,
					args = {
						dkColors = {
							type = 'group',
							name = L["Border Color"],
							order = 7,
							guiInline  = true,
							get = ColorGetter,
		                    set = ColorSetter,
							args = {
								full = {
									type = 'color',
									name = L["Full"],
									order = 1,
								},
								med = {
									type = 'color',
									name = L["Med"],
									order = 2,
								},
								low = {
									type = 'color',
									name = L["Low"],
									order = 3,
								},
							},
						},
					},
				},
				warrior = {
					type = 'group',
					name = L["Warrior"],
					inline = true,
					disabled = selfDisabled,
					args = {
						warSunders = {
							type = 'toggle',
							order = 2,
							name = L["Sunder Armor"],
							desc = L["Track Warrior sunder armors"],
						},
						warSundersTimer = {
							type = 'toggle',
							order = 3,
							name = L["Sunder Armor Timer"],
							desc = L["Track Warrior sunder armors expiration"],
							disabled = function() return not db.warSunders; end,
						},
					},
				},
			},
		}
	end
	return options;
end

local HideAbilities, UpdateCombos, UpdateTargetAuras, OnUpdateAura, RuneOnUpdate
do
	function HideAbilities()

		for i = 1, 5 do
			_G["DHUD3_"..side.."b"..i]:Hide()
			_G["DHUD3_"..side.."b"..i.."_Border"]:SetVertexColor(1, 1, 1);
			_G["DHUD3_"..side.."b"..i.."_Text"]:Hide()
			_G["DHUD3_"..side.."b"..i]:SetScript("OnUpdate", nil)
		end
	end

	function UpdateCombos()

		local points = GetComboPoints("player");
		for i = 1, 5 do
	        if i <= points then
				_G["DHUD3_"..side.."b"..i.."_Back"]:SetTexture("Interface\\AddOns\\DHUD3\\textures\\c"..i);
	            _G["DHUD3_"..side.."b"..i]:Show()
	        else
	            _G["DHUD3_"..side.."b"..i]:Hide()
	        end
		end
	end

	-- Update target auras
    function UpdateTargetAuras(filter, spell, trackTime)

		local name, texture, duration, expirationTime, caster, last;
		local r, g, b, hex, button, timeLeft;
		for i = 1, 40 do
			name, _, texture, count, _, duration, expirationTime, caster = UnitAura("target", i, filter);
			if ( name == spell and caster == "player") then
				for j = 1, 6 do
					if ( j <= count ) then
						button = _G["DHUD3_"..side.."b"..j];
						timeLeft = expirationTime - GetTime();
						if (timeLeft >= button.duration) then
							r, g, b, hex = DHUD3:Colorize(db.colors, 1);
							button.duration = timeLeft;
						else
							r, g, b, hex = DHUD3:Colorize(db.colors, timeLeft/button.duration);
						end
						button.hasAura = true;
						button.id = i;
						button.filter = filter;
						button.auraTimer = 0;
						button.left = timeLeft;
						button.last = false;
						_G["DHUD3_"..side.."b"..j.."_Back"]:SetTexture(texture);
						buffBorder = _G["DHUD3_"..side.."b"..j.."_Border"];
						buffBorder:SetVertexColor(r, g, b);
						buffBorder:Show();
						_G["DHUD3_"..side.."b"..j.."_Text"]:Hide();
						button:SetScript("OnUpdate", OnUpdateAura);
						if ( db.tips ) then
							button:SetScript("OnEnter", function(this)
					                if not this:IsVisible() or ( this:GetEffectiveAlpha() == 0 ) then
					                    return;
					                end
					                GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
					                GameTooltip:SetUnitAura("target", this.id, this.filter);
					            end )
					        button:SetScript("OnLeave", function()
					                GameTooltip:Hide();
					            end )
						end
						button:EnableMouse(true);
						button:Show();
					else
						_G["DHUD3_"..side.."b"..j]:Hide();
						_G["DHUD3_"..side.."b"..j]:EnableMouse(false);
						_G["DHUD3_"..side.."b"..j].duration = 0;
					end
				end
				button = _G["DHUD3_"..side.."b"..count];
				button.last = true;
				if ( trackTime ) then
					button.trackTime = true;
					local buffText = _G["DHUD3_"..side.."b"..count.."_Text"];
					buffText:SetText("|cff"..hex..DHUD3:FormatTime(button.left));
					local w = buffText:GetStringWidth() + 10;
					buffText:SetWidth(w);
					buffText:Show();
				end
				break;
			end
		end
    end

	-- On update event for abilities
	function OnUpdateAura(event, elapsed)
	    this.auraTimer = this.auraTimer + elapsed
	    if this:GetEffectiveAlpha() == 0 then
	        this:EnableMouse(false)
	    else
	        this:EnableMouse(true)
	        if this.auraTimer >= abilityUpdateInterval then
				local timeLeft = 0;
				this.left = this.left - this.auraTimer
				timeLeft = this.left
	            if timeLeft <= 0 then
					this.id = 0
	                this.hasAura = false
	                this:EnableMouse(false)
	                this:SetScript("OnUpdate", nil)
	                return this:Hide()
	            end
				this.auraTimer = 0
	            local r, g, b, hex = DHUD3:Colorize(db.colors, this.left / this.duration);
	            local buffBorder = _G[this:GetName().."_Border"];
	            buffBorder:SetVertexColor(r, g, b);
				if ( this.last and this.trackTime) then
		            local buffText = _G[this:GetName().."_Text"];
		            buffText:SetText("|cff"..hex..DHUD3:FormatTime(this.left));
		            local w = buffText:GetStringWidth() + 10;
		            buffText:SetWidth(w);
				end
				if ( GameTooltip:IsOwned(this) ) then
					GameTooltip:SetUnitAura("target", this.id, this.filter);
				end
	        end
	    end
	end

	function RuneOnUpdate(event, elapsed)
		if ( this.flash ) then
			local fadeAlpha = _G[this:GetName().."_Flash"]:GetAlpha() + 0.15;
			DHUD3:Debug("Flash", fadeAlpha);
	        if fadeAlpha < 1 then
	            _G[this:GetName().."_Flash"]:SetAlpha(fadeAlpha)
	        else
	            this.flash = false
				_G[this:GetName().."_Flash"]:Hide();
				if this.ready then
					this:SetScript("OnUpdate", nil)
				end
	        end
		elseif ( this.ready ) then
			DHUD3:Debug("Rune ready");
			local r, g, b = DHUD3:Colorize(db.dkColors, 0);
			_G[this:GetName().."_Border"]:SetVertexColor(r, g, b);
			this:EnableMouse(false)
			_G[this:GetName().."_Text"]:Hide();
			--Flash
			this.flash = true;
			_G[this:GetName().."_Flash"]:SetAlpha(0)
	        _G[this:GetName().."_Flash"]:Show()
	        _G[this:GetName().."_Flash"].fadeAlpha = 0
		else
			this.auraTimer = this.auraTimer + elapsed
		    if this:GetEffectiveAlpha() == 0 then
		        this:EnableMouse(false)
		    else
		        this:EnableMouse(true)
		        if this.auraTimer >= abilityUpdateInterval then
					local timeLeft = 0;
					this.left = this.left - this.auraTimer
		            if this.left <= 0 then
						this.ready = true;
		            else
						this.auraTimer = 0
			            local r, g, b, hex = DHUD3:Colorize(db.dkColors, this.left / this.duration);
			            local buffBorder = _G[this:GetName().."_Border"];
			            buffBorder:SetVertexColor(r, g, b);
						local buffText = _G[this:GetName().."_Text"];
						buffText:SetText("|cff"..hex..DHUD3:FormatTime(this.left));
						buffText:Show();
						local w = buffText:GetStringWidth() + 10;
						buffText:SetWidth(w);
						if ( GameTooltip:IsOwned(this) ) then
							GameTooltip:SetUnitAura("target", this.id, this.filter);
						end
					end
		        end
		    end
		end
	end

end

function Abilities:OnInitialize()

    self.db = DHUD3.db:RegisterNamespace(MODNAME, defaults);
	db = self.db.profile;
	self:SetEnabledState(DHUD3:GetModuleEnabled(MODNAME));
	DHUD3:RegisterModuleOptions(MODNAME, getOptions, L["Abilities"]);

	local buttons = DHUD3.layout.buttons;
	-- Buttons for Combos/Totems/Sunders/Lacerates
    for k = 1, 6 do
        -- Left
		ref = CreateFrame("Button", "DHUD3_lb"..k, _G["DHUD3_leftAbility"]);
        ref:ClearAllPoints();
        ref:SetPoint("BOTTOM", _G["DHUD3_leftAbility"], "BOTTOM", 0, 0);
        ref:SetHeight(26);
        ref:SetWidth(26);
		bgt = ref:CreateTexture("DHUD3_lb"..k.."_Back", "ARTWORK  ");
        bgt:SetTexture("Interface\\AddOns\\DHUD3\\icon");
        bgt:SetVertexColor(1, 1, 1);
        bgt:SetPoint("CENTER", ref, "CENTER", 0, 0);
        bgt:SetHeight(26);
        bgt:SetWidth(26);
        bgt:SetTexCoord(0, 1, 0, 1);
        bgt:SetBlendMode("BLEND");
		
		bgt = ref:CreateTexture("DHUD3_lb"..k.."_Border", "OVERLAY");
        bgt:SetTexture("Interface\\AddOns\\DHUD3\\textures\\serenity0");
        bgt:SetVertexColor(1, 1, 1);
        bgt:SetPoint("CENTER", ref, "CENTER", 0, 0);
        bgt:SetHeight(26*1.6);
        bgt:SetWidth(26*1.6);
        bgt:SetTexCoord(0, 1, 0, 1);
        bgt:SetBlendMode("BLEND");
		bgt = ref:CreateTexture("DHUD3_lb"..k.."_Flash", "ARTWORK");
		bgt:SetTexture("Interface\\AddOns\\DHUD3\\textures\\runeFlash");
		bgt:SetPoint("CENTER", ref, "CENTER", 0, 0);
        bgt:SetHeight(26*1.6);
        bgt:SetWidth(26*1.6);
        bgt:SetTexCoord(0, 1, 0, 1);
        bgt:SetBlendMode("BLEND");
		bgt:Hide();
        font = ref:CreateFontString("DHUD3_lb"..k.."_Text", "OVERLAY");
        font:SetFontObject(GameFontHighlightSmall);
        font:ClearAllPoints();
        font:SetPoint("CENTER", ref, "CENTER", 0, 0);
        font:SetText("");
        font:SetJustifyH("CENTER");
        font:SetJustifyV("CENTER");
        font:SetWidth(26*2);
		ref.duration = 0;
        ref:Hide();

		-- Right
		ref = CreateFrame("Button", "DHUD3_rb"..k, _G["DHUD3_rightAbility"]);
        ref:ClearAllPoints();
        ref:SetPoint("BOTTOM", _G["DHUD3_rightAbility"], "BOTTOM", 0, 0);
        ref:SetHeight(26);
        ref:SetWidth(26);
		bgt = ref:CreateTexture("DHUD3_rb"..k.."_Border", "BORDER");
        bgt:SetTexture("Interface\\AddOns\\DHUD3\\textures\\serenity0");
        bgt:SetVertexColor(1, 1, 1);
        bgt:SetPoint("CENTER", ref, "CENTER", 0, 0);
        bgt:SetHeight(26*1.5);
        bgt:SetWidth(26*1.5);
        bgt:SetTexCoord(0, 1, 0, 1);
        bgt:SetBlendMode("BLEND");
		bgt = ref:CreateTexture("DHUD3_rb"..k.."_Flash", "ARTWORK");
		bgt:SetTexture("Interface\\AddOns\\DHUD3\\textures\\runeFlash");
		bgt:SetPoint("CENTER", ref, "CENTER", 0, 0);
        bgt:SetHeight(26*1.6);
        bgt:SetWidth(26*1.6);
        bgt:SetTexCoord(0, 1, 0, 1);
        bgt:SetBlendMode("BLEND");
		bgt:Hide();
        font = ref:CreateFontString("DHUD3_rb"..k.."_Text", "OVERLAY");
        font:SetFontObject(GameFontHighlightSmall);
        font:ClearAllPoints();
        font:SetPoint("CENTER", ref, "CENTER", 0, 0);
        font:SetText("");
        font:SetJustifyH("CENTER");
        font:SetJustifyV("CENTER");
        font:SetWidth(26*2);
		ref:SetNormalTexture( "Interface\\AddOns\\DHUD3\\icon" );
		ref.duration = 0;
        ref:Hide();
    end
end

function Abilities:OnEnable()

	DHUD3:Debug("Abilities:OnEnable");
	side = db.side
	self:Refresh();
	if not self:IsEnabled() then return; end

	--Listen for changes in the Auras configiration to change abilities side
	self:RegisterMessage("DHUD3_AURAS_REFRESH");
	-- Listen to bar pops
	self:RegisterMessage("DHUD3_BAR_SHOW");
	self:RegisterMessage("DHUD3_BAR_HIDE");

end

function Abilities:Refresh()

	--DHUD3:Debug("Abilities:Refresh", side);
	db = self.db.profile;
	if not self:IsEnabled() then return; end

	if not abiliSideBlocked then
		side = db.side;
	end
	DHUD3_leftAbility:SetScale(db.scale);
	DHUD3_rightAbility:SetScale(db.scale);
	abiliPosition = "i";
	local hide, parent, xDev;
	if ( side == "l" ) then
		parent = _G["DHUD3_leftAbility"];
		hide = "r";
		xDev = - ( 151 / db.scale) - 13 * db.scale;
	else
		parent = _G["DHUD3_rightAbility"];
		hide = "l";
		xDev = ( 151 / db.scale) + 13 * db.scale;
	end
	local x, y;
	local xCol = 0;
	for i = 1, 6 do
		-- Show
		local name = "DHUD3_"..side.."b"..i;
		local button = _G[name];
		button:ClearAllPoints();
		step = ( 256 * db.scale)/8;
		if (i < 9 ) then
			y = (step * (i - 0.5));
			yPos = (32 * ( i - 0.5)) - 15;
		else
			y = (step * (i - 8.5));
			xCol = (side == "l") and -35 or 35;
			yPos = (32 * ( i - 8.5)) - 15;
		end
		if ( side == "l" ) then
			x = (-17.899 - 0.351*y + 0.00137*y^2)/db.scale;
		else
			x = (17.899 + 0.351*y - 0.00137*y^2)/db.scale;
		end
        button:SetPoint("BOTTOM", parent, "BOTTOM", x + xDev + xCol, yPos);
		button:SetScript("OnUpdate", nil);
		button:Hide();
		font = _G[name.."_Text"]
        font:SetFont(defaultfont, db.fontSize);
        local w = font:GetStringWidth() * 1.1;
        font:SetWidth(w);

		--Hide
		local name = "DHUD3_"..hide.."b"..i;
		button = _G[name];
		button:SetScript("OnUpdate", nil);
		button:SetScript("OnEnter", nil);
		button:Hide();
        font = _G[name.."_Text"];
        font:SetFont(defaultfont, db.fontSize * db.scale)
        w = font:GetStringWidth() * 1.1
        font:SetWidth(w);
	end
	
	local _, class = UnitClass("player");
	if ( class == "DRUID" ) then
		self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
		self:RegisterEvent("PLAYER_TARGET_CHANGED");
		if ( db.druidCombos ) then
			self:RegisterEvent("UNIT_COMBO_POINTS");
		else
			self:UnregisterEvent("UNIT_COMBO_POINTS");
		end
		if ( db.druidLacerates ) then
			self:RegisterEvent("UNIT_AURA");
		else
			self:UnregisterEvent("UNIT_AURA");
		end
		if ( db.druidLifebloom ) then
			self:RegisterEvent("UNIT_AURA");
		else
			self:UnregisterEvent("UNIT_AURA");
		end
		self:PLAYER_TARGET_CHANGED();
	elseif (class == "ROGUE") then
		if ( db.rogueCombos ) then
			self:RegisterEvent("UNIT_COMBO_POINTS");
		else
			self:UnregisterEvent("UNIT_COMBO_POINTS");
		end
		self:PLAYER_TARGET_CHANGED();
	elseif ( class == "WARRIOR" ) then
		self:RegisterEvent("PLAYER_TARGET_CHANGED");
		if ( db.warSunders ) then
			self:RegisterEvent("UNIT_AURA");
		else
			self:UnregisterEvent("UNIT_AURA");
		end
		self:PLAYER_TARGET_CHANGED();
	elseif ( class == "DEATHKNIGHT" ) then
		local runeType, name, button, runeReady;
		for i = 1, 6 do
			runeType = GetRuneType(i);
			_, _, runeReady = GetRuneCooldown(i);
			name = "DHUD3_"..side.."b"..i;
			button = _G[name];
			button.rune = i;
			button.runeType = runeType;
			button.ready = runeReady;
			button:SetNormalTexture(runeTextures[runeType]);
			buffBorder = _G[name.."_Border"];
			buffBorder:SetVertexColor(0, 1, 0);
			button:Show();
		end
		self:RegisterEvent("RUNE_POWER_UPDATE");
		self:RegisterEvent("RUNE_TYPE_UPDATE");
	end
end

function Abilities:UNIT_COMBO_POINTS(event, who)
	
	if ( who == "player") then
		UpdateCombos();
	end
end

function Abilities:UPDATE_SHAPESHIFT_FORM(event, who)

	local powerType = UnitPowerType("player");
	HideAbilities();
	if ( db.druidLifebloom and ( powerType == 0 ) ) then
		-- Is Druid
		UpdateTargetAuras("HELPFUL", "Lifebloom", db.druidLifebloomTimer);
	elseif ( db.druidLacerates and ( powerType == 1 ) ) then
		-- Is Bear
		UpdateTargetAuras("HARMFUL", "Lacerate", db.druidLacerates);
	elseif ( db.druidCombos and ( powerType == 3) ) then
		-- Is Cat
		UpdateCombos();
	end
end

function Abilities:PLAYER_TARGET_CHANGED(event)
	
	HideAbilities();
	if ( UnitExists("target") ) then
		local _, class = UnitClass("player");
		if ( class == "DRUID" ) then
			self:UPDATE_SHAPESHIFT_FORM();
		elseif ( class == "ROGUE" ) then
			UpdateCombos();
		end
		self:UNIT_AURA("TARGET_CHANGED", "target");
	end
end

function Abilities:UNIT_AURA(event, who)

	--DHUD3:Debug("UNIT_AURA", event, who)
    if ( who == "target" ) then
		local _, class = UnitClass("player");
		if ( class == "DRUID" ) then
			local powerType, typeName = UnitPowerType("player");
			if ( db.druidLifebloom and ( powerType == 0 ) ) then
				-- Is Druid
				UpdateTargetAuras("HELPFUL", "Lifebloom", db.druidLifebloomTimer);
			elseif ( db.druidLacerates and ( powerType == 1 ) ) then
				-- Is Bear
				UpdateTargetAuras("HARMFUL", "Lacerate", db.druidLaceratesTimer);
		    end
		elseif ( class == "WARRIOR" ) then
			if ( db.warSunders ) then
				UpdateTargetAuras("HARMFUL", "Sunder Armor", db.warSundersTimer);
			end
		end
    end
end

function Abilities:RUNE_POWER_UPDATE(event, ...)

	-- The rune is usable or unusable
	local rune, usable = ...;
	local j;
	if ( not usable ) then
		--DHUD3:Debug("Move rune", rune, GetRuneType(rune));
		local k = 1;
		-- Search for the ready rune
		while _G["DHUD3_"..side.."b"..k].rune ~= rune do
			k = k + 1;
		end
		for i = k, 5 do
			origin = "DHUD3_"..side.."b"..(i+1);
			dest = "DHUD3_"..side.."b"..(i);
			_G[dest].runeType = _G[origin].runeType;
			_G[dest].ready = _G[origin].ready;
			_G[dest].rune = _G[origin].rune;
			--DHUD3:Debug("Moving rune", i+1,  _G[origin].rune, _G[dest].rune, _G[origin].runeType, _G[dest].runeType);
			_G[dest]:SetNormalTexture(runeTextures[_G[origin].runeType]);
			if ( not _G[dest].ready ) then
				_G[dest].left = _G[origin].left;
				_G[dest].duration = _G[origin].duration;
				_G[dest].auraTimer = abilityUpdateInterval;
				_G[dest]:SetScript("OnUpdate", RuneOnUpdate);
			end
		end
		local button = _G["DHUD3_"..side.."b6"];
		button.rune = rune;
		button.runeType = GetRuneType(rune);
		button:SetNormalTexture(runeTextures[button.runeType]);
		button.ready = false;
		local _, duration = GetRuneCooldown(rune)
		button.left = duration;
		button.duration = duration;
		button.auraTimer = 0;
		button.flash = false;
		local r, g, b, hex = DHUD3:Colorize(db.dkColors, 1);
		buffBorder = _G["DHUD3_"..side.."b6_Border"];
		buffBorder:SetVertexColor(r, g, b);
		local buffText = _G["DHUD3_"..side.."b6_Text"];
		buffText:SetText("|cff"..hex..DHUD3:FormatTime(button.left));
		local w = buffText:GetStringWidth() + 10;
		buffText:SetWidth(w);
		buffText:Show();
		button:SetScript("OnUpdate", RuneOnUpdate);
	else
		for i = 1, 6 do
			if _G["DHUD3_"..side.."b"..(i)].rune == rune then
				_G["DHUD3_"..side.."b"..(i)].ready = true;
			end
		end
	end
end

function Abilities:RUNE_TYPE_UPDATE(event, ...)
	-- Rune type changed
	DHUD3:Debug("RUNE_POWER_UPDATE", rune)
	local rune = ...;
	
end

function Abilities:DHUD3_BAR_SHOW(event, bar)

	--DHUD3:Debug("DHUD3_BAR_SHOW", bar);
	local barSide = string_sub(bar, 1, 1);
	local position = string_sub(bar, 2);
	if ( barSide == side and position ~= abiliPosition ) then
		-- The bar popped in the same side the auras are and the auras need to be moved
		local xDis = 0;
		local move = false;
		if ( abiliPosition == "i" ) then
			if ( position == "m" ) then
				xDis = 15;
				move = true;
				abiliOrigin = abiliPosition;
				abiliPosition = position;
			elseif ( position == "o" ) then
				xDis = 30;
				move = true;
				abiliOrigin = abiliPosition;
				abiliPosition = position;
			end
		elseif ( abiliPosition == "m" ) then
			if ( position == "o" ) then
				xDis = 15;
				move = true;
				abiliOrigin = abiliPosition;
				abiliPosition = position;
			end
		end
		if ( move ) then
			xDis = xDis/db.scale;
			local button, point, relativeTo, relativePoint, xOfs, yOfs;
			for i = 1, 6 do
				button = _G["DHUD3_rb"..i];
				point, relativeTo, relativePoint, xOfs, yOfs = button:GetPoint(1);
				button:ClearAllPoints();
		        button:SetPoint(point, relativeTo, relativePoint, xOfs + xDis, yOfs);
				button = _G["DHUD3_lb"..i];
				point, relativeTo, relativePoint, xOfs, yOfs = button:GetPoint(1);
				button:ClearAllPoints();
		        button:SetPoint(point, relativeTo, relativePoint, xOfs - xDis, yOfs);
			end
		end
	end
end

function Abilities:DHUD3_BAR_HIDE(event, bar)

	--DHUD3:Debug("Abilities:DHUD3_BAR_HIDE", bar, abiliPosition);
	local barSide = string_sub(bar, 1, 1);
	local position = string_sub(bar, 2);
	if ( barSide == side and position == abiliPosition ) then
		-- The bar popped in the same side the auras are and the auras need to be moved
		local xDis = 0;
		local move = false;
		if ( abiliPosition == "m" ) then
				xDis = -15;
				move = true;
				abiliPosition = abiliOrigin;
		elseif ( abiliPosition == "o" ) then
			if ( abiliOrigin == "i" ) then
				xDis = -30;
				move = true;
				abiliPosition = "i";
			elseif ( abiliOrigin == "m" ) then
				xDis = -15;
				move = true;
				abiliPosition = "m";
			end
		end
		if ( move ) then
			xDis = xDis/db.scale;
			local button, point, relativeTo, relativePoint, xOfs, yOfs;
			for i = 1, 6 do
				button = _G["DHUD3_rb"..i];
				point, relativeTo, relativePoint, xOfs, yOfs = button:GetPoint(1);
				button:ClearAllPoints();
		        button:SetPoint(point, relativeTo, relativePoint, xOfs + xDis, yOfs);
				button = _G["DHUD3_lb"..i];
				point, relativeTo, relativePoint, xOfs, yOfs = button:GetPoint(1);
				button:ClearAllPoints();
		        button:SetPoint(point, relativeTo, relativePoint, xOfs - xDis, yOfs);
			end
		end
	elseif ( side == db.side and position ~= abiliPosition ) then
		-- The bar popped but the auras need not to be moved, verify if the origin needs to change
		if ( abiliPosition == "o" and position == "m") then
			abiliOrigin = "i";
		end
	end
end

function Abilities:DHUD3_AURAS_REFRESH(event, enabled, aurasSide)
	
	--DHUD3:Debug("DHUD3_AURAS_REFRESH", event, enabled, aurasSide);
	abiliSideBlocked = enabled;
	if ( enabled ) then
		if ( aurasSide == "l" ) then
			side = "r";
		else
			side = "l";
		end
	else
		side = db.side
	end
	self:Refresh();

end

